@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,11 +20,15 @@ import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.StringJoiner;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import org.codehaus.plexus.compiler.AbstractCompiler;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
@@ -96,8 +100,9 @@ public class JDTCompiler extends AbstractCompiler {
         getLogger().info("Compiling " + sourceFiles.length + " " + "source file" + (sourceFiles.length == 1 ? "" : "s")
                 + " to " + destinationDir.getAbsolutePath());
 
-        Map<String, String> customCompilerArguments = config.getCustomCompilerArgumentsAsMap();
-        checkCompilerArgs(customCompilerArguments, custom);
+        Collection<Map.Entry<String, String>> customCompilerArgumentEntries = config
+                .getCustomCompilerArgumentsEntries();
+        checkCompilerArgs(customCompilerArgumentEntries, custom);
 
         String[] args = buildCompilerArguments(config, custom, sourceFiles);
 
@@ -173,9 +178,7 @@ public class JDTCompiler extends AbstractCompiler {
             args.add(getPathString(sourceLocations));
         }
 
-        for (int i = 0; i < sourceFiles.length; i++) {
-            args.add(sourceFiles[i]);
-        }
+        args.addAll(Arrays.asList(sourceFiles));
 
         if (config.isOptimize()) {
             args.add("-O");
@@ -208,6 +211,10 @@ public class JDTCompiler extends AbstractCompiler {
             args.add("-nowarn");
         }
 
+        if (config.isFailOnWarning()) {
+            args.add("-failOnWarning");
+        }
+
         // TODO: this could be much improved
         if (StringUtils.isEmpty(config.getTargetVersion())) {
             // Required, or it defaults to the target of your JDK (eg 1.5)
@@ -232,17 +239,18 @@ public class JDTCompiler extends AbstractCompiler {
             args.add(config.getSourceEncoding());
         }
 
-        Map<String, String> customCompilerArguments = config.getCustomCompilerArguments();
-        for (Map.Entry<String, String> entry : customCompilerArguments.entrySet()) {
+        Collection<Map.Entry<String, String>> customCompilerArgumentsEntries = config
+                .getCustomCompilerArgumentsEntries();
+        for (Map.Entry<String, String> entry : customCompilerArgumentsEntries) {
 
-            String key = (String) entry.getKey();
+            String key = entry.getKey();
 
             if (StringUtils.isEmpty(key) || key.startsWith("@")) {
                 continue;
             }
 
             if ("use.java.home".equals(key)) {
-                custom.javaHome = (String) entry.getValue();
+                custom.javaHome = entry.getValue();
                 continue;
             }
 
@@ -253,7 +261,7 @@ public class JDTCompiler extends AbstractCompiler {
 
             args.add(key);
 
-            String value = (String) entry.getValue();
+            String value = entry.getValue();
 
             if (StringUtils.isEmpty(value)) {
                 continue;
@@ -262,7 +270,7 @@ public class JDTCompiler extends AbstractCompiler {
             args.add(value);
         }
 
-        return (String[]) args.toArray(new String[args.size()]);
+        return args.toArray(new String[args.size()]);
     }
 
     private static boolean suppressSource(CompilerConfiguration config) {
@@ -311,9 +319,7 @@ public class JDTCompiler extends AbstractCompiler {
             returnCode = CommandLineUtils.executeCommandLine(cli, out, err);
 
             messages = parseModernStream(new BufferedReader(new StringReader(err.getOutput())));
-        } catch (CommandLineException e) {
-            throw new CompilerException("Error while executing the external compiler.", e);
-        } catch (IOException e) {
+        } catch (CommandLineException | IOException e) {
             throw new CompilerException("Error while executing the external compiler.", e);
         }
 
@@ -342,7 +348,7 @@ public class JDTCompiler extends AbstractCompiler {
         StringWriter out = new StringWriter();
         StringWriter err = new StringWriter();
 
-        Main compiler = new Main(new PrintWriter(out), new PrintWriter(err), false);
+        Main compiler = new Main(new PrintWriter(out), new PrintWriter(err), false, null, null);
         compiler.options.put(CompilerOptions.OPTION_ReportForbiddenReference, CompilerOptions.ERROR);
         List<String> jdtCompilerArgs = new ArrayList<>(Arrays.asList(args));
         if (custom.javaHome != null) {
@@ -449,7 +455,7 @@ public class JDTCompiler extends AbstractCompiler {
      *            the given classpath entry
      */
     private String createClasspathArgument(List<String> classpath, CustomCompilerConfiguration custom) {
-        final String[] pathElements = (String[]) classpath.toArray(new String[classpath.size()]);
+        final String[] pathElements = classpath.toArray(new String[classpath.size()]);
 
         // empty path return empty string
         if (pathElements.length == 0) {
@@ -458,35 +464,25 @@ public class JDTCompiler extends AbstractCompiler {
 
         // no access rules, can set the path directly
         if (custom.accessRules == null) {
-            StringBuilder result = new StringBuilder();
-            for (int i = 0; i < pathElements.length; i++) {
-                result.append(pathElements[i]);
-                if (i < pathElements.length - 1) {
-                    result.append(File.pathSeparatorChar);
-                }
-            }
-            return result.toString();
+            return Stream.of(pathElements).collect(Collectors.joining(File.pathSeparator));
         }
 
         int rulesLength = custom.accessRules.size();
-        String[] rules = (String[]) custom.accessRules.toArray(new String[rulesLength]);
+        String[] rules = custom.accessRules.toArray(new String[rulesLength]);
         int nextRule = 0;
-        final StringBuffer result = new StringBuffer();
+        final StringJoiner result = new StringJoiner(File.pathSeparator);
 
         // access rules are expected in the same order as the classpath, but
         // there could
         // be elements in the classpath not in the access rules or access rules
         // not in the classpath
-        for (int i = 0, max = pathElements.length; i < max; i++) {
-            if (i > 0)
-                result.append(File.pathSeparatorChar);
-            String pathElement = pathElements[i];
-            result.append(pathElement);
+        for (String pathElement : pathElements) {
+            result.add(pathElement);
             // the rules list is [path, rule, path, rule, ...]
             for (int j = nextRule; j < rulesLength; j += 2) {
                 String rule = rules[j];
                 if (pathElement.endsWith(rule)) {
-                    result.append(rules[j + 1]);
+                    result.add(rules[j + 1]);
                     nextRule = j + 2;
                     break;
                 }
@@ -499,7 +495,7 @@ public class JDTCompiler extends AbstractCompiler {
                     int ruleLength = rule.length();
                     if (pathElement.regionMatches(false, pathElement.length() - ruleLength + 1, rule, 0,
                             ruleLength - 1)) {
-                        result.append(rules[j + 1]);
+                        result.add(rules[j + 1]);
                         nextRule = j + 2;
                         break;
                     }
@@ -508,7 +504,7 @@ public class JDTCompiler extends AbstractCompiler {
                     // might
                     int ruleLength = rule.length();
                     if (pathElement.regionMatches(false, pathElement.length() - ruleLength - 1, rule, 0, ruleLength)) {
-                        result.append(rules[j + 1]);
+                        result.add(rules[j + 1]);
                         nextRule = j + 2;
                         break;
                     }
@@ -516,19 +512,20 @@ public class JDTCompiler extends AbstractCompiler {
             }
         }
 
-        String s = result.toString();
-        return s;
+        return result.toString();
     }
 
     /**
      * check the compiler arguments. Extract from files specified using @, lines marked with
-     * ADAPTER_PREFIX These lines specify information that needs to be interpreted by us.
+     * {@link #ADAPTER_PREFIX}. These lines specify information that needs to be interpreted by us.
      * 
      * @param args
      *            compiler arguments to process
      */
-    private void checkCompilerArgs(Map<String, String> args, CustomCompilerConfiguration custom) {
-        for (String arg : args.keySet()) {
+    private void checkCompilerArgs(Collection<Map.Entry<String, String>> argEntries,
+            CustomCompilerConfiguration custom) {
+        for (Map.Entry<String, String> argEntry : argEntries) {
+            String arg = argEntry.getKey();
             if (arg.charAt(0) == '@') {
                 try {
                     char[] content = Util.getFileCharContent(new File(arg.substring(1)), null);
